@@ -1,10 +1,10 @@
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: array.c,v 1.137 2004/04/14 22:04:37 mast Exp $
+*/
 
-/*\
-||| This file a part of Pike, and is copyright by Fredrik Hubinette
-||| Pike is distributed as GPL (General Public License)
-||| See the files COPYING and DISCLAIMER for more information.
-\*/
-/**/
 #include "global.h"
 #include "svalue.h"
 #include "array.h"
@@ -15,7 +15,6 @@
 #include "opcodes.h"
 #include "pike_error.h"
 #include "pike_types.h"
-#include "cyclic.h"
 #include "fsort.h"
 #include "builtin_functions.h"
 #include "pike_memory.h"
@@ -24,37 +23,41 @@
 #include "security.h"
 #include "stuff.h"
 #include "bignum.h"
+#include "cyclic.h"
 
-RCSID("$Id: array.c,v 1.110 2003/11/09 02:40:35 mast Exp $");
+RCSID("$Id: array.c,v 1.137 2004/04/14 22:04:37 mast Exp $");
 
 PMOD_EXPORT struct array empty_array=
 {
-  1,                     /* Never free */
-#ifdef PIKE_SECURITY
-  0,
-#endif
+  PIKE_CONSTANT_MEMOBJ_INIT(1), /* Never free */
   &weak_empty_array,     /* Next */
   &weak_shrink_empty_array, /* previous (circular) */
   0,                     /* Size = 0 */
   0,                     /* malloced Size = 0 */
   0,                     /* no types */
   0,			 /* no flags */
+  empty_array.real_item, /* Initialize the item pointer. */
+#ifdef HAVE_UNION_INIT
+  {{0, 0, {0}}}, /* Only to avoid warnings. */
+#endif
 };
 PMOD_EXPORT struct array weak_empty_array=
 {
-  1,
-#ifdef PIKE_SECURITY
-  0,
+  PIKE_CONSTANT_MEMOBJ_INIT(1),
+  &weak_shrink_empty_array, &empty_array, 0, 0, 0, ARRAY_WEAK_FLAG,
+  weak_empty_array.real_item,
+#ifdef HAVE_UNION_INIT
+  {{0, 0, {0}}}, /* Only to avoid warnings. */
 #endif
-  &weak_shrink_empty_array, &empty_array, 0, 0, 0, ARRAY_WEAK_FLAG
 };
 PMOD_EXPORT struct array weak_shrink_empty_array=
 {
-  1,
-#ifdef PIKE_SECURITY
-  0,
+  PIKE_CONSTANT_MEMOBJ_INIT(1),
+  &empty_array, &weak_empty_array, 0, 0, 0, ARRAY_WEAK_FLAG|ARRAY_WEAK_SHRINK,
+  weak_shrink_empty_array.real_item,
+#ifdef HAVE_UNION_INIT
+  {{0, 0, {0}}}, /* Only to avoid warnings. */
 #endif
-  &empty_array, &weak_empty_array, 0, 0, 0, ARRAY_WEAK_FLAG|ARRAY_WEAK_SHRINK
 };
 
 struct array *gc_internal_array = &empty_array;
@@ -71,7 +74,7 @@ PMOD_EXPORT struct array *low_allocate_array(ptrdiff_t size, ptrdiff_t extra_spa
   struct array *v;
   ptrdiff_t e;
 
-  if(size == 0)
+  if(size+extra_space == 0)
   {
     add_ref(&empty_array);
     return &empty_array;
@@ -90,12 +93,11 @@ PMOD_EXPORT struct array *low_allocate_array(ptrdiff_t size, ptrdiff_t extra_spa
   v->flags=0;
 
   v->malloced_size = DO_NOT_WARN((INT32)(size + extra_space));
+  v->item=v->real_item;
   v->size = DO_NOT_WARN((INT32)size);
-  v->refs=1;
+  INIT_PIKE_MEMOBJ(v);
   LINK_ARRAY(v);
 
-  INITIALIZE_PROT(v);
-
   for(e=0;e<v->size;e++)
   {
     ITEM(v)[e].type=T_INT;
@@ -125,7 +127,13 @@ PMOD_EXPORT void really_free_array(struct array *v)
 {
 #ifdef PIKE_DEBUG
   if(v == & empty_array || v == &weak_empty_array || v == &weak_shrink_empty_array)
-    fatal("Tried to free some *_empty_array.\n");
+    Pike_fatal("Tried to free some *_empty_array.\n");
+  if (v->refs) {
+#ifdef DEBUG_MALLOC
+    describe_something(v, T_ARRAY, 0,2,0, NULL);
+#endif
+    Pike_fatal("Freeing array with %d refs.\n", v->refs);
+  }
 #endif
 
 #ifdef PIKE_DEBUG
@@ -133,7 +141,7 @@ PMOD_EXPORT void really_free_array(struct array *v)
 #endif
 
   add_ref(v);
-  FREE_PROT(v);
+  EXIT_PIKE_MEMOBJ(v);
   free_svalues(ITEM(v), v->size, v->type_field);
   v->refs--;
   array_free_no_free(v);
@@ -159,24 +167,12 @@ PMOD_EXPORT struct array *array_set_flags(struct array *a, int flags)
       case ARRAY_WEAK_FLAG|ARRAY_WEAK_SHRINK:
 	add_ref(a = &weak_shrink_empty_array); break;
       default:
-	fatal("Invalid flags %x\n", flags);
+	Pike_fatal("Invalid flags %x\n", flags);
     }
   }
   return a;
 }
 
-/*
- * Extract an svalue from an array
- */
-PMOD_EXPORT void array_index_no_free(struct svalue *s,struct array *v,INT32 index)
-{
-#ifdef PIKE_DEBUG
-  if(index<0 || index>=v->size)
-    fatal("Illegal index in low level index routine.\n");
-#endif
-
-  assign_svalue_no_free(s, ITEM(v) + index);
-}
 
 /*
  * Extract an svalue from an array
@@ -185,7 +181,7 @@ PMOD_EXPORT void array_index(struct svalue *s,struct array *v,INT32 index)
 {
 #ifdef PIKE_DEBUG
   if(index<0 || index>=v->size)
-    fatal("Illegal index in low level index routine.\n");
+    Pike_fatal("Illegal index in low level index routine.\n");
 #endif
 
   add_ref(v);
@@ -244,30 +240,12 @@ PMOD_EXPORT void array_free_index(struct array *v,INT32 index)
 {
 #ifdef PIKE_DEBUG
   if(index<0 || index>=v->size)
-    fatal("Illegal index in low level free index routine.\n");
+    Pike_fatal("Illegal index in low level free index routine.\n");
 #endif
 
   free_svalue(ITEM(v) + index);
 }
 
-/*
- * Set an index in an array
- */
-PMOD_EXPORT void array_set_index(struct array *v,INT32 index, struct svalue *s)
-{
-#ifdef PIKE_DEBUG
-  if(index<0 || index>v->size)
-    fatal("Illegal index in low level array set routine.\n");
-#endif
-
-  add_ref(v);
-  check_destructed(s);
-
-  v->type_field = (v->type_field & ~BIT_UNFINISHED) | (1 << s->type);
-  assign_svalue( ITEM(v) + index, s);
-  free_array(v);
-}
-
 
 PMOD_EXPORT void simple_set_index(struct array *a,struct svalue *ind,struct svalue *s)
 {
@@ -317,7 +295,7 @@ PMOD_EXPORT struct array *array_insert(struct array *v,struct svalue *s,INT32 in
 {
 #ifdef PIKE_DEBUG
   if(index<0 || index>v->size)
-    fatal("Illegal index in low level insert routine.\n");
+    Pike_fatal("Illegal index in low level insert routine.\n");
 #endif
 
   /* Can we fit it into the existing block? */
@@ -383,7 +361,7 @@ PMOD_EXPORT struct array *resize_array(struct array *a, INT32 size)
       struct array *ret;
       ret=low_allocate_array(size, (size>>1) + 4);
       MEMCPY(ITEM(ret),ITEM(a),sizeof(struct svalue)*a->size);
-      ret->type_field = a->type_field | BIT_INT;
+      ret->type_field = DO_NOT_WARN((TYPE_FIELD)(a->type_field | BIT_INT));
       a->size=0;
       free_array(a);
       return ret;
@@ -405,10 +383,10 @@ PMOD_EXPORT struct array *array_shrink(struct array *v, ptrdiff_t size)
 
 #ifdef PIKE_DEBUG
   if(v->refs>2) /* Odd, but has to be two */
-    fatal("Array shrink on array with many references.\n");
+    Pike_fatal("Array shrink on array with many references.\n");
 
   if(size > v->size)
-    fatal("Illegal argument to array_shrink.\n");
+    Pike_fatal("Illegal argument to array_shrink.\n");
 #endif
 
   if(size*2 < v->malloced_size + 4) /* Should we realloc it? */
@@ -437,7 +415,7 @@ PMOD_EXPORT struct array *array_remove(struct array *v,INT32 index)
 
 #ifdef PIKE_DEBUG
   if(index<0 || index >= v->size)
-    fatal("Illegal argument to array_remove.\n");
+    Pike_fatal("Illegal argument to array_remove.\n");
 #endif
 
   array_free_index(v, index);
@@ -479,7 +457,7 @@ PMOD_EXPORT ptrdiff_t array_search(struct array *v, struct svalue *s,
 
 #ifdef PIKE_DEBUG
   if(start<0)
-    fatal("Start of find_index is less than zero.\n");
+    Pike_fatal("Start of find_index is less than zero.\n");
 #endif
 
   check_destructed(s);
@@ -492,7 +470,7 @@ PMOD_EXPORT ptrdiff_t array_search(struct array *v, struct svalue *s,
    * for destructed objects/functions
    */
   if((v->type_field & (1 << s->type))  ||
-     (IS_ZERO(s) && (v->type_field & (BIT_FUNCTION|BIT_OBJECT))) ||
+     (UNSAFE_IS_ZERO(s) && (v->type_field & (BIT_FUNCTION|BIT_OBJECT))) ||
      ( (v->type_field | (1<<s->type))  & BIT_OBJECT )) /* for overloading */
   {
     if(start)
@@ -523,16 +501,32 @@ PMOD_EXPORT struct array *slice_array(struct array *v, ptrdiff_t start,
 
 #ifdef PIKE_DEBUG
   if(start > end || end>v->size || start<0)
-    fatal("Illegal arguments to slice_array()\n");
+    Pike_fatal("Illegal arguments to slice_array()\n");
 
   if(d_flag > 1)  array_check_type_field(v);
 #endif
 
-  if(start==0 && v->refs==1)	/* Can we use the same array? */
+#if 1
+  if(v->refs==1)	/* Can we use the same array? */
   {
-    add_ref(v);
-    return array_shrink(v,end);
+    if((end-start) &&
+       (end-start)*2 >
+       v->malloced_size+4+(v->item-v->real_item) ) /* don't waste too much memory */
+    {
+      add_ref(v);
+      free_svalues(ITEM(v) + end, v->size - end, v->type_field);
+      free_svalues(ITEM(v), start, v->type_field);
+      v->item+=start;
+      v->malloced_size-=start;
+      v->size=end-start;
+#ifdef PIKE_DEBUG
+      if(d_flag>1)
+	check_array(v);
+#endif
+      return v;
+    }
   }
+#endif
 
   a=allocate_array_no_init(end-start,0);
   a->type_field = v->type_field;
@@ -554,7 +548,7 @@ PMOD_EXPORT struct array *friendly_slice_array(struct array *v,
 
 #ifdef PIKE_DEBUG
   if(start > end || end>v->size || start<0)
-    fatal("Illegal arguments to slice_array()\n");
+    Pike_fatal("Illegal arguments to slice_array()\n");
 
   if(d_flag > 1)  array_check_type_field(v);
 #endif
@@ -660,7 +654,10 @@ static int internal_cmpfun(INT32 *a,
 			   cmpfun current_cmpfun,
 			   struct svalue *current_array_p)
 {
-  return current_cmpfun(current_array_p + *a, current_array_p + *b);
+  int res = current_cmpfun(current_array_p + *a, current_array_p + *b);
+  /* If the comparison considers the elements equal we compare their
+   * positions. Thus we get a stable sort function. */
+  return res ? res : *a - *b;
 }
 
 #define CMP(X,Y) internal_cmpfun((X),(Y),current_cmpfun, current_array_p)
@@ -675,6 +672,7 @@ static int internal_cmpfun(INT32 *a,
 #undef EXTRA_ARGS
 #undef XARGS
 
+/* The sort is stable. */
 INT32 *get_order(struct array *v, cmpfun fun)
 {
   INT32 e, *current_order;
@@ -695,9 +693,9 @@ INT32 *get_order(struct array *v, cmpfun fun)
   return current_order;
 }
 
-static INLINE int set_svalue_cmpfun(struct svalue *a, struct svalue *b)
+INLINE int set_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
 {
-  INT32 def,fun;
+  INT32 def;
   if(a->type == b->type)
   {
     switch(a->type)
@@ -753,7 +751,7 @@ static INLINE int set_svalue_cmpfun(struct svalue *a, struct svalue *b)
     if (FIND_LFUN(a->u.object->prog,LFUN_LT) != -1) {
       push_svalue(b);
       apply_lfun(a->u.object,LFUN_LT,1);
-      if(!IS_ZERO(sp-1))
+      if(!UNSAFE_IS_ZERO(Pike_sp-1))
       {
 	pop_stack();
 	return -1;
@@ -763,7 +761,7 @@ static INLINE int set_svalue_cmpfun(struct svalue *a, struct svalue *b)
     if (FIND_LFUN(a->u.object->prog,LFUN_GT) != -1) {
       push_svalue(b);
       apply_lfun(a->u.object,LFUN_GT,1);
-      if(!IS_ZERO(sp-1))
+      if(!UNSAFE_IS_ZERO(Pike_sp-1))
       {
 	pop_stack();
 	return 1;
@@ -773,7 +771,7 @@ static INLINE int set_svalue_cmpfun(struct svalue *a, struct svalue *b)
     if (FIND_LFUN(a->u.object->prog,LFUN_EQ) != -1) {
       push_svalue(b);
       apply_lfun(a->u.object,LFUN_EQ,1);
-      if (!IS_ZERO(sp-1)) {
+      if (!UNSAFE_IS_ZERO(Pike_sp-1)) {
 	pop_stack();
 	return 0;
       }
@@ -784,7 +782,7 @@ static INLINE int set_svalue_cmpfun(struct svalue *a, struct svalue *b)
     if (FIND_LFUN(b->u.object->prog,LFUN_LT) != -1) {
       push_svalue(a);
       apply_lfun(b->u.object,LFUN_LT,1);
-      if(!IS_ZERO(sp-1))
+      if(!UNSAFE_IS_ZERO(Pike_sp-1))
       {
 	pop_stack();
 	return 1;
@@ -794,7 +792,7 @@ static INLINE int set_svalue_cmpfun(struct svalue *a, struct svalue *b)
     if (FIND_LFUN(b->u.object->prog,LFUN_GT) != -1) {
       push_svalue(a);
       apply_lfun(b->u.object,LFUN_GT,1);
-      if(!IS_ZERO(sp-1))
+      if(!UNSAFE_IS_ZERO(Pike_sp-1))
       {
 	pop_stack();
 	return -1;
@@ -804,7 +802,7 @@ static INLINE int set_svalue_cmpfun(struct svalue *a, struct svalue *b)
     if (FIND_LFUN(b->u.object->prog,LFUN_EQ) != -1) {
       push_svalue(a);
       apply_lfun(b->u.object,LFUN_EQ,1);
-      if (!IS_ZERO(sp-1)) {
+      if (!UNSAFE_IS_ZERO(Pike_sp-1)) {
 	pop_stack();
 	return 0;
       }
@@ -815,7 +813,7 @@ static INLINE int set_svalue_cmpfun(struct svalue *a, struct svalue *b)
   return def;
 }
 
-static int switch_svalue_cmpfun(struct svalue *a, struct svalue *b)
+static int switch_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
 {
   if(a->type == b->type)
   {
@@ -842,7 +840,7 @@ static int switch_svalue_cmpfun(struct svalue *a, struct svalue *b)
   }
 }
 
-static int alpha_svalue_cmpfun(struct svalue *a, struct svalue *b)
+static int alpha_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
 {
   if(a->type == b->type)
   {
@@ -1040,7 +1038,7 @@ PMOD_EXPORT void array_fix_type_field(struct array *v)
   if(t & ~(v->type_field))
   {
     describe(v);
-    fatal("Type field out of order!\n");
+    Pike_fatal("Type field out of order!\n");
   }
 #endif
   v->type_field = t;
@@ -1061,7 +1059,7 @@ void array_check_type_field(struct array *v)
   for(e=0; e<v->size; e++)
   {
     if(ITEM(v)[e].type > MAX_TYPE)
-      fatal("Type is out of range.\n");
+      Pike_fatal("Type is out of range.\n");
       
     t |= 1 << ITEM(v)[e].type;
   }
@@ -1069,7 +1067,7 @@ void array_check_type_field(struct array *v)
   if(t & ~(v->type_field))
   {
     describe(v);
-    fatal("Type field out of order!\n");
+    Pike_fatal("Type field out of order!\n");
   }
 }
 #endif
@@ -1214,6 +1212,51 @@ PMOD_EXPORT struct array *add_arrays(struct svalue *argp, INT32 args)
   for(size=e=0;e<args;e++)
     size+=argp[e].u.array->size;
 
+#if 1
+  {
+    INT32 tmp=0;
+    for(e=0;e<args;e++)
+    {
+      v=argp[e].u.array;
+      if(v->refs == 1 &&
+	 (v->item - v->real_item) >= tmp &&
+	 v->malloced_size >= size - tmp)
+      {
+	debug_malloc_touch(v);
+	argp[e].type=T_INT;
+	for(tmp=e-1;tmp>=0;tmp--)
+	{
+	  debug_malloc_touch(argp[tmp].u.array);
+	  v->type_field|=argp[tmp].u.array->type_field;
+	  assign_svalues_no_free(ITEM(v) - argp[tmp].u.array->size,
+				 ITEM(argp[tmp].u.array),
+				 argp[tmp].u.array->size,
+				 argp[tmp].u.array->type_field);
+	  v->item-=argp[tmp].u.array->size;
+	  v->malloced_size+=argp[tmp].u.array->size;
+	}
+
+	for(tmp=e+1;tmp<args;tmp++)
+	{
+	  debug_malloc_touch(argp[tmp].u.array);
+	  v->type_field|=argp[tmp].u.array->type_field;
+	  assign_svalues_no_free(ITEM(v) + v->size,
+				 ITEM(argp[tmp].u.array),
+				 argp[tmp].u.array->size,
+				 argp[tmp].u.array->type_field);
+	  v->size+=argp[tmp].u.array->size;
+	}
+#ifdef PIKE_DEBUG
+	if(d_flag>1)
+	  check_array(v);
+#endif
+	return v;
+      }
+      tmp+=v->size;
+    }
+  }
+#endif
+
   if(args && argp[0].u.array->refs==1)
   {
     e=argp[0].u.array->size;
@@ -1564,7 +1607,6 @@ int array_is_constant(struct array *a,
 			      p);
 }
 
-
 node *make_node_from_array(struct array *a)
 {
   struct svalue s;
@@ -1576,6 +1618,7 @@ node *make_node_from_array(struct array *a)
     
   if(a->type_field == BIT_INT)
   {
+    debug_malloc_touch(a);
     for(e=0; e<a->size; e++)
       if(ITEM(a)[e].u.integer != 0)
 	break;
@@ -1584,9 +1627,11 @@ node *make_node_from_array(struct array *a)
       return mkefuncallnode("allocate",mkintnode(a->size));
     }
   }
+  debug_malloc_touch(a);
   if(!is_more_than_one_bit(a->type_field))
   {
     e=0;
+    debug_malloc_touch(a);
     switch(a->type_field)
     {
       case BIT_INT:
@@ -1612,6 +1657,7 @@ node *make_node_from_array(struct array *a)
 	    break;
 	break;
     }
+    debug_malloc_touch(a);
     if(e == a->size)
       return mkefuncallnode("allocate",mknode(F_ARG_LIST,
 					      mkintnode(a->size),
@@ -1620,12 +1666,14 @@ node *make_node_from_array(struct array *a)
   
   if(array_is_constant(a,0))
   {
+    debug_malloc_touch(a);
     s.type=T_ARRAY;
     s.subtype=0;
     s.u.array=a;
     return mkconstantsvaluenode(&s);
   }else{
     node *ret=0;
+    debug_malloc_touch(a);
     for(e=0; e<a->size; e++)
       ret=mknode(F_ARG_LIST,ret,mksvaluenode(ITEM(a)+e));
     return mkefuncallnode("aggregate",ret);
@@ -1779,7 +1827,7 @@ PMOD_EXPORT struct array *explode(struct pike_string *str,
       case 1: f=(explode_searchfunc)mojt.vtab->func1; break;
       case 2: f=(explode_searchfunc)mojt.vtab->func2; break;
 #ifdef PIKE_DEBUG
-      default: fatal("Illegal shift.\n");
+      default: Pike_fatal("Illegal shift.\n");
 #endif
     }
 
@@ -1898,8 +1946,7 @@ PMOD_EXPORT struct array *copy_array_recursively(struct array *a,struct processi
 PMOD_EXPORT void apply_array(struct array *a, INT32 args)
 {
   INT32 e;
-  struct array *ret;
-  struct svalue *argp = Pike_sp - args;
+  struct svalue *argp = Pike_sp-args;
   TYPE_FIELD new_types = 0;
   struct array *cycl;
   DECLARE_CYCLIC();
@@ -1961,7 +2008,7 @@ PMOD_EXPORT struct array *reverse_array(struct array *a)
   return ret;
 }
 
-PMOD_EXPORT void array_replace(struct array *a,
+void array_replace(struct array *a,
 		   struct svalue *from,
 		   struct svalue *to)
 {
@@ -1976,18 +2023,33 @@ PMOD_EXPORT void check_array(struct array *a)
   INT32 e;
 
   if(a->next->prev != a)
-    fatal("Array check: a->next->prev != a\n");
+    Pike_fatal("Array check: a->next->prev != a\n");
 
   if(a->size > a->malloced_size)
-    fatal("Array is larger than malloced block!\n");
+    Pike_fatal("Array is larger than malloced block!\n");
+
+  if(a->size < 0)
+    Pike_fatal("Array size is negative!\n");
+
+  if(a->malloced_size < 0)
+    Pike_fatal("Array malloced size is negative!\n");
+
+  if(a->item < a->real_item)
+  {
+#ifdef DEBUG_MALLOC
+    describe(a);
+#endif
+    Pike_fatal("Array item pointer is too small!\n");
+  }
 
   if(a->refs <=0 )
-    fatal("Array has zero refs.\n");
+    Pike_fatal("Array has zero refs.\n");
+
 
   for(e=0;e<a->size;e++)
   {
     if(! ( (1 << ITEM(a)[e].type) & (a->type_field) ) && ITEM(a)[e].type<16)
-      fatal("Type field lies.\n");
+      Pike_fatal("Type field lies.\n");
     
     check_svalue(ITEM(a)+e);
   }
@@ -2004,7 +2066,7 @@ void check_all_arrays(void)
 
     a=a->next;
     if(!a)
-      fatal("Null pointer in array list.\n");
+      Pike_fatal("Null pointer in array list.\n");
   } while (a != & empty_array);
 }
 #endif /* PIKE_DEBUG */
@@ -2025,11 +2087,10 @@ static void gc_check_array(struct array *a)
 
 void gc_mark_array_as_referenced(struct array *a)
 {
-  int e;
   if(gc_mark(a)) {
 #ifdef PIKE_DEBUG
     if (a == &empty_array || a == &weak_empty_array || a == &weak_shrink_empty_array)
-      fatal("Trying to gc mark some *_empty_array.\n");
+      Pike_fatal("Trying to gc mark some *_empty_array.\n");
 #endif
 
     if (a == gc_mark_array_pos)
@@ -2051,7 +2112,7 @@ void gc_mark_array_as_referenced(struct array *a)
 	  int d=0;
 #ifdef PIKE_DEBUG
 	  if (a->refs != 1)
-	    fatal("Got %d refs to weak shrink array "
+	    Pike_fatal("Got %d refs to weak shrink array "
 		  "which we'd like to change the size on.\n", a->refs);
 #endif
 	  t = 0;
@@ -2094,10 +2155,9 @@ void gc_mark_array_as_referenced(struct array *a)
 void real_gc_cycle_check_array(struct array *a, int weak)
 {
   GC_CYCLE_ENTER(a, weak) {
-    int e;
 #ifdef PIKE_DEBUG
     if (a == &empty_array || a == &weak_empty_array || a == &weak_shrink_empty_array)
-      fatal("Trying to gc cycle check some *_empty_array.\n");
+      Pike_fatal("Trying to gc cycle check some *_empty_array.\n");
 #endif
 
     if (a->type_field & BIT_COMPLEX)
@@ -2131,7 +2191,7 @@ unsigned gc_touch_all_arrays(void)
     debug_gc_touch(a);
     n++;
     if (!a->next || a->next->prev != a)
-      fatal("Error in array link list.\n");
+      Pike_fatal("Error in array link list.\n");
     a=a->next;
   } while (a != &empty_array);
   return n;
@@ -2161,7 +2221,7 @@ void gc_mark_all_arrays(void)
   while (gc_mark_array_pos != &empty_array) {
     struct array *a = gc_mark_array_pos;
 #ifdef PIKE_DEBUG
-    if (!a) fatal("Null pointer in array list.\n");
+    if (!a) Pike_fatal("Null pointer in array list.\n");
 #endif
     gc_mark_array_pos = a->next;
     if(gc_is_referenced(a))
@@ -2197,7 +2257,7 @@ void gc_free_all_unreferenced_arrays(void)
   {
 #ifdef PIKE_DEBUG
     if (!a)
-      fatal("Null pointer in array list.\n");
+      Pike_fatal("Null pointer in array list.\n");
 #endif
     if(gc_do_free(a))
     {
@@ -2257,8 +2317,7 @@ void zap_all_arrays(void)
 {
   struct array *a,*next;
 
-  a=&empty_array;
-  do
+  for(a=empty_array.next;a!=&empty_array;a=next)
   {
 
 #if defined(PIKE_DEBUG) && defined(DEBUG_MALLOC)
@@ -2270,12 +2329,9 @@ void zap_all_arrays(void)
     add_ref(a);
     free_svalues(ITEM(a), a->size, a->type_field);
     a->size=0;
-    
-    if(!(next=a->next))
-      fatal("Null pointer in array list.\n");
 
     SET_NEXT_AND_FREE(a,free_array);
-  } while (a != & empty_array);
+  }
 }
 
 
@@ -2295,10 +2351,10 @@ void count_memory_in_arrays(INT32 *num_, INT32 *size_)
 
 PMOD_EXPORT struct array *explode_array(struct array *a, struct array *b)
 {
-  INT32 e,d,q,start;
+  INT32 e,d,start;
   struct array *tmp;
 
-  q=start=0;
+  start=0;
 #if 0
   if(!a->size)
   {
@@ -2307,38 +2363,43 @@ PMOD_EXPORT struct array *explode_array(struct array *a, struct array *b)
 #endif
   if(b->size)
   {
-    for(e=0;e<=a->size - b->size;e++)
-    {
-      for(d=0;d<b->size;d++)
+    BEGIN_AGGREGATE_ARRAY(1) {
+      for(e=0;e<=a->size - b->size;e++)
       {
-	if(!is_eq(ITEM(a)+(e+d),ITEM(b)+d))
-	  break;
+	for(d=0;d<b->size;d++)
+	{
+	  if(!is_eq(ITEM(a)+(e+d),ITEM(b)+d))
+	    break;
+	}
+	if(d==b->size)
+	{
+	  check_stack(1);
+	  push_array(friendly_slice_array(a, start, e));
+	  DO_AGGREGATE_ARRAY(120);
+	  e+=b->size-1;
+	  start=e+1;
+	}
       }
-      if(d==b->size)
-      {
-	check_stack(1);
-	push_array(friendly_slice_array(a, start, e));
-	q++;
-	e+=b->size-1;
-	start=e+1;
-      }
-    }
-    check_stack(1);
-    push_array(friendly_slice_array(a, start, a->size));
-    q++;
+      check_stack(1);
+      push_array(friendly_slice_array(a, start, a->size));
+    } END_AGGREGATE_ARRAY;
   }else{
-    check_stack(a->size);
-    for(e=0;e<a->size;e++) push_array(friendly_slice_array(a, e, e+1));
-    q=a->size;
+    check_stack(120);
+    BEGIN_AGGREGATE_ARRAY(a->size) {
+      for(e=0;e<a->size;e++) {
+	push_array(friendly_slice_array(a, e, e+1));
+	DO_AGGREGATE_ARRAY(120);
+      }
+    } END_AGGREGATE_ARRAY;
   }
-  tmp=aggregate_array(q);
+  tmp=(--Pike_sp)->u.array;
   if(tmp->size) tmp->type_field=BIT_ARRAY;
   return tmp;
 }
 
 PMOD_EXPORT struct array *implode_array(struct array *a, struct array *b)
 {
-  INT32 e,size;
+  INT32 e, size;
   struct array *ret;
 
   if (!a->size) {
@@ -2377,7 +2438,7 @@ PMOD_EXPORT struct array *implode_array(struct array *a, struct array *b)
   }
 #ifdef PIKE_DEBUG
   if(size != ret->size)
-    fatal("Implode_array failed miserably\n");
+    Pike_fatal("Implode_array failed miserably (%d != %d)\n", size, ret->size);
 #endif
   return ret;
 }
