@@ -1,7 +1,7 @@
 /*
  * Created by
- * $Id: make_ci.pike,v 1.7 2001/06/21 13:24:49 grubba Exp $
- * on Sun Jul 25 03:34:14 2004
+ * $Id: make_ci.pike,v 1.7 2001/06/21 13:23:20 grubba Exp $
+ * on Tue Jun 15 20:29:02 2004
  *
  * Table used for looking up the case of
  * Unicode characters.
@@ -56,7 +56,7 @@
 { 0x00019a, CIM_NONE, 0x0000, },
 { 0x00019c, CIM_UPPERDELTA, 0x00d3, },
 { 0x00019d, CIM_UPPERDELTA, 0x00d5, },
-{ 0x00019e, CIM_NONE, 0x0000, },
+{ 0x00019e, CIM_LOWERDELTA, -0x0082, },
 { 0x00019f, CIM_UPPERDELTA, 0x00d6, },
 { 0x0001a0, CIM_CASEBIT, 0x0001, },
 { 0x0001a6, CIM_UPPERDELTA, 0x00da, },
@@ -89,7 +89,8 @@
 { 0x0001f6, CIM_UPPERDELTA, -0x0061, },
 { 0x0001f7, CIM_UPPERDELTA, -0x0038, },
 { 0x0001f8, CIM_CASEBIT, 0x0001, },
-{ 0x000220, CIM_NONE, 0x0000, },
+{ 0x000220, CIM_UPPERDELTA, -0x0082, },
+{ 0x000221, CIM_NONE, 0x0000, },
 { 0x000222, CIM_CASEBIT, 0x0001, },
 { 0x000234, CIM_NONE, 0x0000, },
 { 0x000253, CIM_LOWERDELTA, 0x00d2, },
@@ -154,7 +155,7 @@
 { 0x0003d5, CIM_LOWERDELTA, 0x002f, },
 { 0x0003d6, CIM_LOWERDELTA, 0x0036, },
 { 0x0003d7, CIM_NONE, 0x0000, },
-{ 0x0003da, CIM_CASEBIT, 0x0001, },
+{ 0x0003d8, CIM_CASEBIT, 0x0001, },
 { 0x0003f0, CIM_LOWERDELTA, 0x0056, },
 { 0x0003f1, CIM_LOWERDELTA, 0x0050, },
 { 0x0003f2, CIM_LOWERDELTA, 0x004f, },
@@ -170,18 +171,16 @@
 { 0x000450, CIM_LOWERDELTA, 0x0050, },
 { 0x000460, CIM_CASEBIT, 0x0001, },
 { 0x000482, CIM_NONE, 0x0000, },
-{ 0x00048c, CIM_CASEBIT, 0x0001, },
+{ 0x00048a, CIM_CASEBIT, 0x0001, },
 { 0x0004c0, CIM_NONE, 0x0000, },
 { 0x0004c1, CIM_CASEBITOFF, 0x0001, },
-{ 0x0004c5, CIM_NONE, 0x0000, },
-{ 0x0004c7, CIM_CASEBITOFF, 0x0001, },
-{ 0x0004c9, CIM_NONE, 0x0000, },
-{ 0x0004cb, CIM_CASEBITOFF, 0x0001, },
-{ 0x0004cd, CIM_NONE, 0x0000, },
+{ 0x0004cf, CIM_NONE, 0x0000, },
 { 0x0004d0, CIM_CASEBIT, 0x0001, },
 { 0x0004f6, CIM_NONE, 0x0000, },
 { 0x0004f8, CIM_CASEBIT, 0x0001, },
 { 0x0004fa, CIM_NONE, 0x0000, },
+{ 0x000500, CIM_CASEBIT, 0x0001, },
+{ 0x000510, CIM_NONE, 0x0000, },
 { 0x000531, CIM_UPPERDELTA, 0x0030, },
 { 0x000557, CIM_NONE, 0x0000, },
 { 0x000561, CIM_LOWERDELTA, 0x0030, },
