@@ -71,42 +71,43 @@
      TOK_FOR = 297,
      TOK_FUNCTION_ID = 298,
      TOK_GAUGE = 299,
-     TOK_IDENTIFIER = 300,
-     TOK_IF = 301,
-     TOK_IMPORT = 302,
-     TOK_INHERIT = 303,
-     TOK_INLINE = 304,
-     TOK_LOCAL_ID = 305,
-     TOK_FINAL_ID = 306,
-     TOK_INT_ID = 307,
-     TOK_LAMBDA = 308,
-     TOK_MULTISET_ID = 309,
-     TOK_MULTISET_END = 310,
-     TOK_MULTISET_START = 311,
-     TOK_LSH_EQ = 312,
-     TOK_MAPPING_ID = 313,
-     TOK_MIXED_ID = 314,
-     TOK_MOD_EQ = 315,
-     TOK_MULT_EQ = 316,
-     TOK_NO_MASK = 317,
-     TOK_OBJECT_ID = 318,
-     TOK_OR_EQ = 319,
-     TOK_PRIVATE = 320,
-     TOK_PROGRAM_ID = 321,
-     TOK_PROTECTED = 322,
-     TOK_PREDEF = 323,
-     TOK_PUBLIC = 324,
-     TOK_RSH_EQ = 325,
-     TOK_STATIC = 326,
-     TOK_STRING_ID = 327,
-     TOK_SUB_EQ = 328,
-     TOK_TYPEDEF = 329,
-     TOK_TYPEOF = 330,
-     TOK_VARIANT = 331,
-     TOK_VOID_ID = 332,
-     TOK_WHILE = 333,
-     TOK_XOR_EQ = 334,
-     TOK_OPTIONAL = 335
+     TOK_GLOBAL = 300,
+     TOK_IDENTIFIER = 301,
+     TOK_IF = 302,
+     TOK_IMPORT = 303,
+     TOK_INHERIT = 304,
+     TOK_INLINE = 305,
+     TOK_LOCAL_ID = 306,
+     TOK_FINAL_ID = 307,
+     TOK_INT_ID = 308,
+     TOK_LAMBDA = 309,
+     TOK_MULTISET_ID = 310,
+     TOK_MULTISET_END = 311,
+     TOK_MULTISET_START = 312,
+     TOK_LSH_EQ = 313,
+     TOK_MAPPING_ID = 314,
+     TOK_MIXED_ID = 315,
+     TOK_MOD_EQ = 316,
+     TOK_MULT_EQ = 317,
+     TOK_NO_MASK = 318,
+     TOK_OBJECT_ID = 319,
+     TOK_OR_EQ = 320,
+     TOK_PRIVATE = 321,
+     TOK_PROGRAM_ID = 322,
+     TOK_PROTECTED = 323,
+     TOK_PREDEF = 324,
+     TOK_PUBLIC = 325,
+     TOK_RSH_EQ = 326,
+     TOK_STATIC = 327,
+     TOK_STRING_ID = 328,
+     TOK_SUB_EQ = 329,
+     TOK_TYPEDEF = 330,
+     TOK_TYPEOF = 331,
+     TOK_VARIANT = 332,
+     TOK_VOID_ID = 333,
+     TOK_WHILE = 334,
+     TOK_XOR_EQ = 335,
+     TOK_OPTIONAL = 336
    };
 #endif
 #define TOK_ARROW 258
@@ -151,56 +152,58 @@
 #define TOK_FOR 297
 #define TOK_FUNCTION_ID 298
 #define TOK_GAUGE 299
-#define TOK_IDENTIFIER 300
-#define TOK_IF 301
-#define TOK_IMPORT 302
-#define TOK_INHERIT 303
-#define TOK_INLINE 304
-#define TOK_LOCAL_ID 305
-#define TOK_FINAL_ID 306
-#define TOK_INT_ID 307
-#define TOK_LAMBDA 308
-#define TOK_MULTISET_ID 309
-#define TOK_MULTISET_END 310
-#define TOK_MULTISET_START 311
-#define TOK_LSH_EQ 312
-#define TOK_MAPPING_ID 313
-#define TOK_MIXED_ID 314
-#define TOK_MOD_EQ 315
-#define TOK_MULT_EQ 316
-#define TOK_NO_MASK 317
-#define TOK_OBJECT_ID 318
-#define TOK_OR_EQ 319
-#define TOK_PRIVATE 320
-#define TOK_PROGRAM_ID 321
-#define TOK_PROTECTED 322
-#define TOK_PREDEF 323
-#define TOK_PUBLIC 324
-#define TOK_RSH_EQ 325
-#define TOK_STATIC 326
-#define TOK_STRING_ID 327
-#define TOK_SUB_EQ 328
-#define TOK_TYPEDEF 329
-#define TOK_TYPEOF 330
-#define TOK_VARIANT 331
-#define TOK_VOID_ID 332
-#define TOK_WHILE 333
-#define TOK_XOR_EQ 334
-#define TOK_OPTIONAL 335
+#define TOK_GLOBAL 300
+#define TOK_IDENTIFIER 301
+#define TOK_IF 302
+#define TOK_IMPORT 303
+#define TOK_INHERIT 304
+#define TOK_INLINE 305
+#define TOK_LOCAL_ID 306
+#define TOK_FINAL_ID 307
+#define TOK_INT_ID 308
+#define TOK_LAMBDA 309
+#define TOK_MULTISET_ID 310
+#define TOK_MULTISET_END 311
+#define TOK_MULTISET_START 312
+#define TOK_LSH_EQ 313
+#define TOK_MAPPING_ID 314
+#define TOK_MIXED_ID 315
+#define TOK_MOD_EQ 316
+#define TOK_MULT_EQ 317
+#define TOK_NO_MASK 318
+#define TOK_OBJECT_ID 319
+#define TOK_OR_EQ 320
+#define TOK_PRIVATE 321
+#define TOK_PROGRAM_ID 322
+#define TOK_PROTECTED 323
+#define TOK_PREDEF 324
+#define TOK_PUBLIC 325
+#define TOK_RSH_EQ 326
+#define TOK_STATIC 327
+#define TOK_STRING_ID 328
+#define TOK_SUB_EQ 329
+#define TOK_TYPEDEF 330
+#define TOK_TYPEOF 331
+#define TOK_VARIANT 332
+#define TOK_VOID_ID 333
+#define TOK_WHILE 334
+#define TOK_XOR_EQ 335
+#define TOK_OPTIONAL 336
 
 
 
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 169 "language.yacc"
+#line 176 "/tmp/pikedeb.b138767487/7.4/src/language.yacc"
 typedef union YYSTYPE {
   int number;
   FLOAT_TYPE fnum;
   struct node_s *n;
   char *str;
+  void *ptr;
 } YYSTYPE;
 /* Line 1240 of yacc.c.  */
-#line 204 "y.tab.h"
+#line 207 "y.tab.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
