@@ -1,9 +1,12 @@
-/* $Id: image_gif.c,v 1.4 2000/12/13 21:16:48 hubbe Exp $ */
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: image_gif.c,v 1.14 2002/10/21 17:06:25 marcus Exp $
+*/
 
 /*
 **! module Image
-**! note
-**!	$Id: image_gif.c,v 1.4 2000/12/13 21:16:48 hubbe Exp $
 **! submodule GIF
 **!
 **!	This submodule keep the GIF encode/decode capabilities
@@ -26,6 +29,7 @@
 **! see also: Image, Image.Image, Image.Colortable
 */
 #include "global.h"
+#include "module.h"
 
 #include "config.h"
 
@@ -35,7 +39,7 @@
 #include <ctype.h>
 
 #include "stralloc.h"
-RCSID("$Id: image_gif.c,v 1.4 2000/12/13 21:16:48 hubbe Exp $");
+RCSID("$Id: image_gif.c,v 1.14 2002/10/21 17:06:25 marcus Exp $");
 #include "pike_macros.h"
 #include "object.h"
 #include "constants.h"
@@ -57,21 +61,24 @@ RCSID("$Id: image_gif.c,v 1.4 2000/12/13 21:16:48 hubbe Exp $");
 
 #include "gif_lzw.h"
 
-/* MUST BE INCLUDED LAST */
-#include "module_magic.h"
+#endif /* WITH_GIF */
+
+#ifdef WITH_GIF
+
+#define sp Pike_sp
 
 #ifdef DYNAMIC_MODULE
 static struct program *image_program = NULL;
 static struct program *image_colortable_program = NULL;
 static struct program *image_layer_program = NULL;
 
-#ifdef FAKE_DYNAMIC_LOAD
+/*  #ifdef FAKE_DYNAMIC_LOAD */
 
 /* These should really be cached in local, static variables */
 #define image_lay ((void(*)(INT32))PIKE_MODULE_IMPORT(Image,image_lay))
 
 #define image_colortable_write_rgb \
- ((void(*)(struct neo_colortable *,unsigned char *))PIKE_MODULE_IMPORT(Image,image_colorable_write_rgb))
+ ((void(*)(struct neo_colortable *,unsigned char *))PIKE_MODULE_IMPORT(Image,image_colortable_write_rgb))
 
 #define image_colortable_size \
   ((ptrdiff_t(*)(struct neo_colortable *))PIKE_MODULE_IMPORT(Image,image_colortable_size))
@@ -82,7 +89,7 @@ static struct program *image_layer_program = NULL;
 #define image_colortable_internal_floyd_steinberg \
   ((void(*)(struct neo_colortable *))PIKE_MODULE_IMPORT(Image,image_colortable_internal_floyd_steinberg))
 
-#endif
+/*  #endif */
 
 #else
 extern struct program *image_program;
@@ -336,16 +343,16 @@ void image_gif_end_block(INT32 args)
 **! arg int disposal
 **!	Disposal method number;
 **!	<dl compact>
-**!	<dt>0<dd>No disposal specified. The decoder is
-**!	         not required to take any action.
-**!	<dt>1<dd>Do not dispose. The graphic is to be left
-**!	         in place.
-**!     <dt>2<dd>Restore to background color. The area used by the
-**!              graphic must be restored to the background color.
-**!     <dt>3<dd>Restore to previous. The decoder is required to
+**!	<dt>0</dt><dd>No disposal specified. The decoder is
+**!	         not required to take any action.</dd>
+**!	<dt>1</dt><dd>Do not dispose. The graphic is to be left
+**!	         in place.</dd>
+**!     <dt>2</dt><dd>Restore to background color. The area used by the
+**!              graphic must be restored to the background color.</dd>
+**!     <dt>3</dt><dd>Restore to previous. The decoder is required to
 **!              restore the area overwritten by the graphic with
-**!              what was there prior to rendering the graphic.
-**!     <dt compact>4-7<dd>To be defined.
+**!              what was there prior to rendering the graphic.</dd>
+**!     <dt compact>4-7</dt><dd>To be defined.</dd>
 **!     </dl>
 **!
 **! see also: _render_block, render_block
@@ -557,7 +564,7 @@ CHRONO("gif _render_block push of packed data begin");
       else
       {
 	 ps=begin_shared_string(lzw.outpos-i+2);
-	 ps->str[0]=lzw.outpos-i;
+	 ps->str[0] = DO_NOT_WARN((char)(lzw.outpos-i));
 	 MEMCPY(ps->str+1,lzw.out+i,lzw.outpos-i);
 	 ps->str[lzw.outpos-i+1]=0;
 	 push_string(end_shared_string(ps));
@@ -609,8 +616,8 @@ CHRONO("gif _render_block end");
 **!	</pre>
 **!
 **!	<illustration type=image/gif>
-**!	object nct=colortable(lena(),32,({({0,0,0})}));
-**!	string s=GIF.header_block(lena()->xsize(),lena()->ysize(),nct);
+**!	object nct=Image.colortable(lena(),32,({({0,0,0})}));
+**!	string s=Image.GIF.header_block(lena()->xsize(),lena()->ysize(),nct);
 **!	foreach ( ({lena()->xsize(),
 **!		    (int)(lena()->xsize()*0.75),
 **!		    (int)(lena()->xsize()*0.5),
@@ -623,14 +630,14 @@ CHRONO("gif _render_block end");
 **!	   object o=lena()->scale(xsize,lena()->ysize());
 **!	   object p=lena()->clear(0,0,0);
 **!	   p->paste(o,(lena()->xsize()-o->xsize())/2,0);
-**!	   s+=GIF.render_block(p,nct,0,0,0,25);
+**!	   s+=Image.GIF.render_block(p,nct,0,0,0,25);
 **!	}
-**!	s+=GIF.netscape_loop_block(200);
-**!	s+=GIF.end_block();
+**!	s+=Image.GIF.netscape_loop_block(200);
+**!	s+=Image.GIF.end_block();
 **!	return s;
 **!	</illustration>The above animation is thus created:
 **!	<pre>
-**!	object nct=colortable(lena,32,({({0,0,0})}));
+**!	object nct=Image.colortable(lena,32,({({0,0,0})}));
 **!	string s=GIF.header_block(lena->xsize(),lena->ysize(),nct);
 **!	foreach ( ({lena->xsize(),
 **!		    (int)(lena->xsize()*0.75),
@@ -644,10 +651,10 @@ CHRONO("gif _render_block end");
 **!	   object o=lena->scale(xsize,lena->ysize());
 **!	   object p=lena->clear(0,0,0);
 **!	   p->paste(o,(lena->xsize()-o->xsize())/2,0);
-**!	   s+=GIF.render_block(p,nct,0,0,0,25);
+**!	   s+=Image.GIF.render_block(p,nct,0,0,0,25);
 **!	}
-**!	s+=GIF.netscape_loop_block(200);
-**!	s+=GIF.end_block();
+**!	s+=Image.GIF.netscape_loop_block(200);
+**!	s+=Image.GIF.end_block();
 **!	write(s);
 **!	</pre>
 **!
@@ -679,16 +686,16 @@ CHRONO("gif _render_block end");
 **! arg int disposal
 **!	Disposal method number;
 **!	<dl compact>
-**!	<dt>0<dd>No disposal specified. The decoder is
-**!	         not required to take any action. (default)
-**!	<dt>1<dd>Do not dispose. The graphic is to be left
-**!	         in place.
-**!     <dt>2<dd>Restore to background color. The area used by the
-**!              graphic must be restored to the background color.
-**!     <dt>3<dd>Restore to previous. The decoder is required to
+**!	<dt>0</dt><dd>No disposal specified. The decoder is
+**!	         not required to take any action. (default)</dd>
+**!	<dt>1</dt><dd>Do not dispose. The graphic is to be left
+**!	         in place.</dd>
+**!     <dt>2</dt><dd>Restore to background color. The area used by the
+**!              graphic must be restored to the background color.</dd>
+**!     <dt>3</dt><dd>Restore to previous. The decoder is required to
 **!              restore the area overwritten by the graphic with
-**!              what was there prior to rendering the graphic.
-**!     <dt compact>4-7<dd>To be defined.
+**!              what was there prior to rendering the graphic.</dd>
+**!     <dt compact>4-7</dt><dd>To be defined.</dd>
 **!     </dl>
 **!
 **! see also: encode, header_block, end_block
@@ -2130,8 +2137,7 @@ void image_gif_decode(INT32 args)
 void image_gif_decode_layers(INT32 args)
 {
    struct array *a,*b;
-   struct image *img,*src,*alpha;
-   struct object *o;
+   struct image *src,*alpha;
    int n;
    int numlayers=0;
 
@@ -2493,10 +2499,10 @@ static void image_gif_lzw_encode(INT32 args)
    image_gif_lzw_init(&lzw,8);
    if (lzw.broken) Pike_error("out of memory\n");
 
-   if (args>=2 && !IS_ZERO(sp+1-args))
+   if (args>=2 && !UNSAFE_IS_ZERO(sp+1-args))
       lzw.earlychange=1;
 
-   if (args>=3 && !IS_ZERO(sp+2-args))
+   if (args>=3 && !UNSAFE_IS_ZERO(sp+2-args))
       lzw.reversebits=1;
    
    image_gif_lzw_add(&lzw,
@@ -2531,9 +2537,9 @@ static void image_gif_lzw_decode(INT32 args)
    s=(unsigned char*)sp[-args].u.string->str;
    len = (ptrdiff_t)sp[-args].u.string->len;
 
-   if (args>=2 && !IS_ZERO(sp+1-args))
+   if (args>=2 && !UNSAFE_IS_ZERO(sp+1-args))
       earlychange=1;
-   if (args>=3 && !IS_ZERO(sp+2-args))
+   if (args>=3 && !UNSAFE_IS_ZERO(sp+2-args))
       reversebits=1;
 
    if (len<1)
@@ -2562,8 +2568,11 @@ static void image_gif_lzw_decode(INT32 args)
    }
    dest=dest0; dlen=dlen0;
 
-   for (n=0; n<clearcode; n++)
-      c[n].prev=0xffff,c[n].len=1,c[n].c=n;
+   for (n=0; n<clearcode; n++) {
+      c[n].prev=0xffff;
+      c[n].len=1;
+      c[n].c = DO_NOT_WARN((unsigned short)n);
+   }
    c[clearcode].len=0; 
    c[endcode].len=0;   
 
@@ -2604,7 +2613,7 @@ static void image_gif_lzw_decode(INT32 args)
 
       if (n==m) 
       {
-	 c[n].prev=last;
+	 c[n].prev = DO_NOT_WARN((unsigned short)last);
 	 c[n].c=c[last].c;
 	 c[n].len=c[last].len+1;
       }
@@ -2676,7 +2685,7 @@ static void image_gif_lzw_decode(INT32 args)
 
 	 if (last!=clearcode)
 	 {
-	    c[m].prev=last;
+	    c[m].prev = DO_NOT_WARN((unsigned short)last);
 	    c[m].len=c[last].len+1;
 	    c[m].c=lc;
 	 }
@@ -2726,7 +2735,7 @@ static void image_gif_lzw_decode(INT32 args)
 
 struct program *image_encoding_gif_program=NULL;
 
-void pike_module_init(void)
+PIKE_MODULE_INIT
 {
 #ifdef DYNAMIC_MODULE
   /* These could be re-written to use PIKE_MODULE_IMPORT */
@@ -2815,11 +2824,11 @@ void pike_module_init(void)
 }
 
 #else /* !WITH_GIF */
-void pike_module_init(void)
+PIKE_MODULE_INIT
 {
 }
 #endif /* WITH_GIF */
 
-void pike_module_exit(void)
+PIKE_MODULE_EXIT
 {
 }
