@@ -1,6 +1,11 @@
 /*
- * $Id: preprocessor.h,v 1.39 2001/07/02 20:32:55 mast Exp $
- *
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: preprocessor.h,v 1.56 2004/04/22 15:45:02 grubba Exp $
+*/
+
+/*
  * Preprocessor template.
  * Based on cpp.c 1.45
  *
@@ -28,6 +33,7 @@
 
 #define lower_cpp		lower_cpp0
 #define find_end_parenthesis	find_end_parenthesis0
+#define find_end_brace		find_end_brace0
 #define PUSH_STRING		PUSH_STRING0
 #define WC_BINARY_FINDSTRING(X, Y)	binary_findstring((char *)X, Y)
 
@@ -62,6 +68,7 @@
 
 #define lower_cpp		lower_cpp1
 #define find_end_parenthesis	find_end_parenthesis1
+#define find_end_brace		find_end_brace1
 #define PUSH_STRING		PUSH_STRING1
 #define WC_BINARY_FINDSTRING	binary_findstring1
 
@@ -90,6 +97,7 @@
 
 #define lower_cpp		lower_cpp2
 #define find_end_parenthesis	find_end_parenthesis2
+#define find_end_brace		find_end_brace2
 #define PUSH_STRING		PUSH_STRING2
 #define WC_BINARY_FINDSTRING	binary_findstring2
 
@@ -206,7 +214,7 @@ static void PUSH_STRING_SHIFT(void *str, ptrdiff_t len, int shift,
     PUSH_STRING2((p_wchar2 *)str, len, buf);
     break;
   default:
-    fatal("cpp(): Bad string shift detected in PUSH_STRING_SHIFT(): %d\n",
+    Pike_fatal("cpp(): Bad string shift detected in PUSH_STRING_SHIFT(): %d\n",
 	  shift);
     break;
   }
@@ -290,11 +298,15 @@ static INLINE ptrdiff_t find_end_parenthesis(struct cpp *this,
 					     ptrdiff_t len,
 					     ptrdiff_t pos)/* pos of first " */
 {
+  INT32 start_line = this->current_line;
   while(1)
   {
     if(pos+1>=len)
     {
+      INT32 save_line = this->current_line;
+      this->current_line = start_line;
       cpp_error(this, "End of file while looking for end parenthesis.");
+      this->current_line = save_line;
       return pos;
     }
 
@@ -305,6 +317,48 @@ static INLINE ptrdiff_t find_end_parenthesis(struct cpp *this,
     case '"':  FIND_END_OF_STRING();  break;
     case '(':  pos=find_end_parenthesis(this, data, len, pos); break;
     case ')':  return pos;
+    case '/':
+      if (data[pos] == '*') {
+	pos++;
+	SKIPCOMMENT();
+      } else if (data[pos] == '/') {
+	pos++;
+	FIND_EOL();
+      }
+    }
+  }
+}
+
+static INLINE ptrdiff_t find_end_brace(struct cpp *this,
+				       WCHAR *data,
+				       ptrdiff_t len,
+				       ptrdiff_t pos)/* pos of first " */
+{
+  INT32 start_line = this->current_line;
+  while(1)
+  {
+    if(pos+1>=len)
+    {
+      INT32 save_line = this->current_line;
+      this->current_line = start_line;
+      cpp_error(this, "End of file while looking for end brace.");
+      this->current_line = save_line;
+      return pos;
+    }
+
+    switch(data[pos++])
+    {
+    case '\n': PUTNL(); this->current_line++; break;
+    case '\'': FIND_END_OF_CHAR();  break;
+    case '"':  FIND_END_OF_STRING();  break;
+    case '{':  pos=find_end_brace(this, data, len, pos); break;
+    case '}':  return pos;
+    case '/':
+      if (data[pos] == '*') {
+	SKIPCOMMENT();
+      } else if (data[pos] == '/') {
+	FIND_EOL();
+      }
     }
   }
 }
@@ -347,7 +401,7 @@ static ptrdiff_t calcC(struct cpp *this, WCHAR *data, ptrdiff_t len,
     l = STRTOL_PCHARP(p, &p2, 0);
     if(COMPARE_PCHARP(p1,>,p2))
     {
-      push_float(f);
+      push_float(DO_NOT_WARN((FLOAT_TYPE)f));
       pos = ((WCHAR *)p1.ptr) - data;
     }else{
       push_int(l);
@@ -674,7 +728,7 @@ static ptrdiff_t calc3(struct cpp *this, WCHAR *data, ptrdiff_t len,
     /* DUMPPOS("inside calc3"); */
 
     check_destructed(Pike_sp-1);
-    if(IS_ZERO(Pike_sp-1))
+    if(UNSAFE_IS_ZERO(Pike_sp-1))
     {
       pos=calc4(this,data,len,pos);
       pop_stack();
@@ -701,7 +755,7 @@ static ptrdiff_t calc2(struct cpp *this, WCHAR *data, ptrdiff_t len,
     /* DUMPPOS("inside calc2"); */
 
     check_destructed(Pike_sp-1);
-    if(!IS_ZERO(Pike_sp-1))
+    if(!UNSAFE_IS_ZERO(Pike_sp-1))
     {
       pos=calc3(this,data,len,pos);
       pop_stack();
@@ -730,7 +784,7 @@ static ptrdiff_t calc1(struct cpp *this, WCHAR *data, ptrdiff_t len,
     pos=calc1(this,data,len,pos);
 
     check_destructed(Pike_sp-3);
-    assign_svalue(Pike_sp-3,IS_ZERO(Pike_sp-3)?Pike_sp-1:Pike_sp-2);
+    assign_svalue(Pike_sp-3,UNSAFE_IS_ZERO(Pike_sp-3)?Pike_sp-1:Pike_sp-2);
     pop_n_elems(2);
   }
   return pos;
@@ -746,22 +800,18 @@ static ptrdiff_t calc(struct cpp *this, WCHAR *data, ptrdiff_t len,
 
   if (SETJMP(recovery))
   {
-    pos=tmp;
-    if(throw_value.type == PIKE_T_ARRAY && throw_value.u.array->size)
-    {
-      union anything *a;
-      a=low_array_get_item_ptr(throw_value.u.array, 0, PIKE_T_STRING);
-      if(a)
-      {
-	cpp_error(this, a->string->str);
-      }else{
-	cpp_error(this, "Nonstandard error format.");
-      }
-    }else{
-      cpp_error(this, "Nonstandard error format.");
-    }
-    free_svalue(&throw_value);
+    struct svalue thrown = throw_value;
     throw_value.type = T_INT;
+
+    cpp_error(this, "Error evaluating expression.");
+
+    push_svalue(&thrown);
+    low_safe_apply_handler("compile_exception", error_handler, compat_handler, 1);
+    if (SAFE_IS_ZERO(sp-1)) cpp_describe_exception(this, &thrown);
+    pop_stack();
+    free_svalue(&thrown);
+
+    pos=tmp;
     FIND_EOL();
     push_int(0);
   }else{
@@ -784,6 +834,8 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 {
   ptrdiff_t pos, tmp, e;
   int tmp2;
+  INT32 first_line = this->current_line;
+  /* FIXME: What about this->current_file? */
   
   for(pos=0; pos<len;)
   {
@@ -882,7 +934,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	if(d && !(d->inside & 1))
 	{
 	  int arg=0;
-	  INT32 startline = this->current_line;
+	  INT32 start_line = this->current_line;
 	  struct string_builder tmp;
 	  struct define_argument arguments [MAX_ARGS];
 	  short inside = d->inside;
@@ -895,10 +947,10 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	    
 	    if(!GOBBLE('('))
 	    {
-	      char buffer[1024];
-	      sprintf(buffer, "Missing ( in the macro %.950s.", 
-		      d->link.s->str);
-	      cpp_error(this, buffer);
+	      if (s) {
+	        string_builder_shared_strcat(&this->buf,s);
+		free_string(s);
+	      }
 	      break;
 	    }
 	    
@@ -918,11 +970,14 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		    arguments[arg].len=0;
 		    continue;
 		  }else{
-		    char buffer[1024];
-		    sprintf(buffer,
-			    "Too few arguments to macro %.950s, expected %d.",
-			    d->link.s->str, d->args);
-		    cpp_error(this, buffer);
+		    if (!d->link.s->size_shift)
+		      cpp_error_sprintf(this,
+					"Too few arguments to macro %s, expected %d.",
+					d->link.s->str, d->args);
+		    else
+		      cpp_error_sprintf(this,
+					"Too few arguments to macro, expected %d.",
+					d->args);
 		    break;
 		  }
 		}
@@ -934,7 +989,10 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	      {
 		if(pos+1>len)
 		{
+		  INT32 save_line = this->current_line;
+		  this->current_line = start_line;
 		  cpp_error(this, "End of file in macro call.");
+		  this->current_line = save_line;
 		  break;
 		}
 		
@@ -960,6 +1018,10 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		case '(':
 		  pos=find_end_parenthesis(this, data, len, pos);
 		  continue;
+
+		case '{':
+		  pos=find_end_brace(this, data, len, pos);
+		  continue;
 		  
 		case ',':
 		  if(d->varargs && arg+1 == d->args) continue;
@@ -975,11 +1037,12 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	    }
 	    SKIPWHITE();
 	    if(!GOBBLE(')')) {
-	      char buffer[1024];
-	      sprintf(buffer, "Missing ) in the macro %.950s.", 
-		      d->link.s->str);
-	      this->current_line = startline;
-	      cpp_error(this, buffer);
+	      this->current_line = start_line;
+	      if (!d->link.s->size_shift)
+		cpp_error_sprintf(this, "Missing ) in the macro %s.",
+				  d->link.s->str);
+	      else
+		cpp_error(this, "Missing ) in macro.");
 	    }
 	  }
 	  
@@ -1007,7 +1070,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 #ifdef PIKE_DEBUG
 	      if (arguments[d->parts[e].argument&DEF_ARG_MASK].arg.shift !=
 		  SHIFT) {
-		fatal("cpp(): Bad argument shift %d != %d\n", SHIFT,
+		Pike_fatal("cpp(): Bad argument shift %d != %d\n", SHIFT,
 		      arguments[d->parts[e].argument&DEF_ARG_MASK].arg.shift);
 	      }
 #endif /* PIKE_DEBUG */
@@ -1020,7 +1083,53 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	      
 	      if(d->parts[e].argument & DEF_ARG_STRINGIFY)
 	      {
-		PUSH_STRING(a,l,&tmp);
+		/* NOTE: At entry a[0] is non white-space. */
+		int e = 0;
+		string_builder_putchar(&tmp, '"');
+		for(e=0; e<l;) {
+		  if (WC_ISSPACE(a[e]) || a[e]=='"' || a[e]=='\\') {
+		    if (e) {
+		      string_builder_append(&tmp, MKPCHARP(a, SHIFT), e);
+		    }
+		    if (a[e] == '"' || a[e]=='\\') {
+		      /* String or quote. */
+		      string_builder_putchar(&tmp, '\\');
+		      string_builder_putchar(&tmp, a[e]);
+		      if (a[e] == '"') {
+			for (e++; (e < l) && (a[e] != '"'); e++) {
+			  string_builder_putchar(&tmp, a[e]);
+			  if (a[e] == '\\') {
+			    string_builder_putchar(&tmp, '\\');
+			    string_builder_putchar(&tmp, a[++e]);
+			    if (a[e] == '\\') {
+			      string_builder_putchar(&tmp, '\\');
+			    }
+			  }
+			}
+			string_builder_putchar(&tmp, '\\');
+			string_builder_putchar(&tmp, '"');
+			e++;
+		      }
+		    } else {
+		      /* White space. */
+		      while ((e < l) && WC_ISSPACE(a[e])) {
+			e++;
+		      }
+		      if (e != l) {
+			string_builder_putchar(&tmp, ' ');
+		      }
+		    }
+		    a += e;
+		    l -= e;
+		    e = 0;
+		  } else {
+		    e++;
+		  }
+		}
+		if (l) {
+		  string_builder_append(&tmp, MKPCHARP(a, SHIFT), l);
+		}
+		string_builder_putchar(&tmp, '"');
 	      }else{
 		if(DEF_ARG_NOPRESPACE)
 		  while(l && WC_ISSPACE(*a))
@@ -1057,10 +1166,24 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	    }
 	  }
 	  
-	  /* FIXME */
-	  for(e=0; e< (ptrdiff_t)tmp.s->len; e++)
-	    if(tmp.s->str[e]=='\n')
-	      tmp.s->str[e]=' ';
+	  /* Remove any newlines from the completed expression. */
+	  switch (tmp.s->size_shift) {
+	  case 0:
+	    for(e=0; e< (ptrdiff_t)tmp.s->len; e++)
+	      if(STR0(tmp.s)[e]=='\n')
+		STR0(tmp.s)[e]=' ';
+	    break;
+	  case 1:
+	    for(e=0; e< (ptrdiff_t)tmp.s->len; e++)
+	      if(STR1(tmp.s)[e]=='\n')
+		STR1(tmp.s)[e]=' ';
+	    break;
+	  case 2:
+	    for(e=0; e< (ptrdiff_t)tmp.s->len; e++)
+	      if(STR2(tmp.s)[e]=='\n')
+		STR2(tmp.s)[e]=' ';
+	    break;
+	  }
 
 	  if(s) d->inside=1;
 	  
@@ -1068,7 +1191,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  tmp.s->len--;
 #ifdef PIKE_DEBUG
 	  if (tmp.s->size_shift != SHIFT) {
-	    fatal("cpp(): Bad shift in #if: %d != %d\n",
+	    Pike_fatal("cpp(): Bad shift in #if: %d != %d\n",
 		  tmp.s->size_shift, SHIFT);
 	  }
 #endif /* PIKE_DEBUG */
@@ -1199,7 +1322,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 			   this->current_file->len,&this->buf);
 	      break;
 	    default:
-	      fatal("cpp(): Filename has bad shift %d\n",
+	      Pike_fatal("cpp(): Filename has bad shift %d\n",
 		    this->current_file->size_shift);
 	      break;
 	  }
@@ -1254,6 +1377,8 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  SKIPSPACE();
 	  
 	  check_stack(3);
+
+	  /* FIXME: Ought to macro-expand (Bug 2440). */
 	  
 	  switch(data[pos++])
 	  {
@@ -1463,11 +1588,11 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	  calc_2(this, (p_wchar2 *)tmp.s->str, tmp.s->len, 0);
 	  break;
 	default:
-	  fatal("cpp(): Bad shift: %d\n", tmp.s->size_shift);
+	  Pike_fatal("cpp(): Bad shift: %d\n", tmp.s->size_shift);
 	  break;
 	}
 	free_string_builder(&tmp);
-	if(IS_ZERO(Pike_sp-1)) nflags|=CPP_NO_OUTPUT;
+	if(SAFE_IS_ZERO(Pike_sp-1)) nflags|=CPP_NO_OUTPUT;
 	pop_stack();
 	pos += lower_cpp(this, data+pos, len-pos, nflags,
 			 auto_convert, charset);
@@ -1592,11 +1717,11 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	    calc_2(this, (p_wchar2 *)tmp.s->str, tmp.s->len,0);
 	    break;
 	  default:
-	    fatal("cpp(): Bad shift: %d\n", tmp.s->size_shift);
+	    Pike_fatal("cpp(): Bad shift: %d\n", tmp.s->size_shift);
 	    break;
 	  }
 	  free_string_builder(&tmp);
-	  if(!IS_ZERO(Pike_sp-1)) flags&=~CPP_NO_OUTPUT;
+	  if(!SAFE_IS_ZERO(Pike_sp-1)) flags&=~CPP_NO_OUTPUT;
 	  pop_stack();
 	}else{
 	  FIND_EOL();
@@ -1768,7 +1893,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 		{
 #ifdef PIKE_DEBUG
 		  if(Pike_sp[-1].type != PIKE_T_INT)
-		    fatal("Internal error in CPP\n");
+		    Pike_fatal("Internal error in CPP\n");
 #endif
 		  Pike_sp[-1].u.integer|=DEF_ARG_NOPOSTSPACE;
 		}
@@ -1871,10 +1996,10 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 	    {
 #ifdef PIKE_DEBUG
 	      if(partbase[e*2+1].type != PIKE_T_INT)
-		fatal("Cpp internal error, expected integer!\n");
+		Pike_fatal("Cpp internal error, expected integer!\n");
 	      
 	      if(partbase[e*2+2].type != PIKE_T_STRING)
-		fatal("Cpp internal error, expected string!\n");
+		Pike_fatal("Cpp internal error, expected string!\n");
 #endif
 	      def->parts[e].argument=partbase[e*2+1].u.integer;
 	      copy_shared_string(def->parts[e].postfix,
@@ -1884,7 +2009,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 #ifdef PIKE_DEBUG
 	    if(def->num_parts==1 &&
 	       (def->parts[0].argument & DEF_ARG_MASK) > MAX_ARGS)
-	      fatal("Internal error in define\n");
+	      Pike_fatal("Internal error in define\n");
 #endif	  
 	    {
 	      struct define *d;
@@ -2063,8 +2188,12 @@ static ptrdiff_t lower_cpp(struct cpp *this,
     }
   }
 
-  if(flags & CPP_EXPECT_ENDIF)
+  if(flags & CPP_EXPECT_ENDIF) {
+    INT32 saved_line = this->current_line;
+    this->current_line = first_line;
     cpp_error(this, "End of file while searching for #endif.");
+    this->current_line = saved_line;
+  }
 
   return pos;
 }
@@ -2083,6 +2212,7 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 
 #undef lower_cpp
 #undef find_end_parenthesis
+#undef find_end_brace
 
 #undef calc
 #undef calc1
@@ -2098,4 +2228,3 @@ static ptrdiff_t lower_cpp(struct cpp *this,
 #undef calcA
 #undef calcB
 #undef calcC
-
